@@ -1,4 +1,3 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/rendition/rendition.c,v 1.58 2003/11/03 05:11:26 tsi Exp $ */
 /*
  * Copyright (C) 1998 The XFree86 Project, Inc.  All Rights Reserved.
  *
@@ -64,6 +63,14 @@
 #include "rendition_shadow.h"
 #include "vbe.h"
 
+#ifdef XSERVER_LIBPCIACCESS
+# include <pciaccess.h>
+# define DEVICE_ID(p)  (p)->device_id
+#else
+# define DEVICE_ID(p)  (p)->chipType
+#endif
+
+
 /*
  * defines
  */
@@ -72,10 +79,10 @@
 
 #define RENDITION_NAME            "RENDITION"
 #define RENDITION_DRIVER_NAME     "rendition"
-#define RENDITION_VERSION_NAME    "4.1.0"
-#define RENDITION_VERSION_MAJOR   4
-#define RENDITION_VERSION_MINOR   1
-#define RENDITION_PATCHLEVEL      0
+#define RENDITION_VERSION_NAME    PACKAGE_VERSION
+#define RENDITION_VERSION_MAJOR   PACKAGE_VERSION_MAJOR
+#define RENDITION_VERSION_MINOR   PACKAGE_VERSION_MINOR
+#define RENDITION_PATCHLEVEL      PACKAGE_VERSION_PATCHLEVEL
 #define RENDITION_VERSION_CURRENT ((RENDITION_VERSION_MAJOR << 24) | \
                  (RENDITION_VERSION_MINOR << 16) | RENDITION_PATCHLEVEL)
 
@@ -101,7 +108,12 @@ static const int MAX_VTOTAL   = 2184;
 
 static const OptionInfoRec * renditionAvailableOptions(int, int);
 static void       renditionIdentify(int);
+#ifdef XSERVER_LIBPCIACCESS
+static Bool renditionPciProbe(DriverPtr drv, int entity_num,
+    struct pci_device *dev, intptr_t match_data);
+#else
 static Bool       renditionProbe(DriverPtr, int);
+#endif
 static Bool       renditionPreInit(ScrnInfoPtr, int);
 static Bool       renditionScreenInit(int, ScreenPtr, int, char **);
 static Bool       renditionSwitchMode(int, DisplayModePtr, int);
@@ -120,6 +132,7 @@ static unsigned int renditionDDC1Read (ScrnInfoPtr pScreenInfo);
 static xf86MonPtr renditionProbeDDC(ScrnInfoPtr pScrn, int index);
 
 static void renditionLoadPalette(ScrnInfoPtr, int, int *, LOCO *, VisualPtr);
+static renditionPtr renditionGetRec(ScrnInfoPtr pScreenInfo);
 
 
 /* 
@@ -137,82 +150,56 @@ OptionInfoRec const renditionOptions[]={
     { -1,                NULL,      OPTV_NONE,    {0}, FALSE }
 };
 
+enum renditionTypes {
+    CHIP_RENDITION_V1000,
+    CHIP_RENDITION_V2x00
+};
+
+/* supported chipsets */
+static SymTabRec renditionChipsets[] = {
+    {CHIP_RENDITION_V1000, "V1000"},
+    {CHIP_RENDITION_V2x00, "V2x00"},
+    {-1,                   NULL}
+};
+
+#ifdef XSERVER_LIBPCIACCESS
+#define RENDITION_DEVICE_MATCH(d, i) \
+    { 0x1163, (d), PCI_MATCH_ANY, PCI_MATCH_ANY, 0, 0, (i) }
+
+static const struct pci_id_match rendition_device_match[] = {
+    RENDITION_DEVICE_MATCH(PCI_CHIP_V1000, CHIP_RENDITION_V1000),
+    RENDITION_DEVICE_MATCH(PCI_CHIP_V2x00, CHIP_RENDITION_V2x00),
+
+    { 0, 0, 0 }
+};
+#else
+static PciChipsets renditionPCIchipsets[] = {
+  { CHIP_RENDITION_V1000, PCI_CHIP_V1000, RES_SHARED_VGA },
+  { CHIP_RENDITION_V2x00, PCI_CHIP_V2x00, RES_SHARED_VGA },
+  { -1,                   -1,             RES_UNDEFINED }
+};
+#endif
+
 _X_EXPORT DriverRec RENDITION={
     RENDITION_VERSION_CURRENT,
     "rendition",
     renditionIdentify,
+#ifdef XSERVER_LIBPCIACCESS
+    NULL,
+#else
     renditionProbe,
+#endif
     renditionAvailableOptions,
     NULL,
-    0
-};
+    0,
+    NULL,
 
-static const char *vgahwSymbols[]={
-    "vgaHWBlankScreen",
-    "vgaHWDPMSSet",
-    "vgaHWFreeHWRec",
-    "vgaHWGetHWRec",
-    "vgaHWGetIOBase",
-    "vgaHWGetIndex",
-    "vgaHWLock",
-    "vgaHWMapMem",
-    "vgaHWProtect",
-    "vgaHWRestore",
-    "vgaHWSave",
-    "vgaHWSaveScreen",
-    "vgaHWUnlock",
-    "vgaHWHandleColormaps",
-    NULL
-};
-
-static const char *ramdacSymbols[] = {
-    "xf86CreateCursorInfoRec",
-    "xf86DestroyCursorInfoRec",
-    "xf86InitCursor",
-    NULL
-};
-
-#if defined(XFree86LOADER) || USE_ACCEL
-static const char *xaaSymbols[] = {
-    "XAACreateInfoRec",
-    "XAADestroyInfoRec",
-    "XAAInit",
-    NULL
-};
+#ifdef XSERVER_LIBPCIACCESS
+    rendition_device_match,
+    renditionPciProbe
 #endif
-
-static const char *ddcSymbols[] = {
-    "xf86DoEDID_DDC1",
-    "xf86PrintEDID",
-    NULL
 };
 
-static const char *int10Symbols[] = {
-    "xf86FreeInt10",
-    "xf86InitInt10",
-    NULL
-};
-
-static const char *fbSymbols[]={
-    "fbScreenInit",
-    "fbPictureInit",
-    NULL
-};
-
-static const char *shadowfbSymbols[] = {
-    "ShadowFBInit",
-    NULL
-};
-
-static const char *vbeSymbols[] = {
-    "VBEInit",
-    "vbeDoEDID",
-    "vbeFree",
-    NULL
-};
-
-
-
 #ifdef XFree86LOADER
 
 /* Module loader interface */
@@ -239,19 +226,16 @@ static pointer
 renditionSetup(pointer Module, pointer Options, int *ErrorMajor, 
                int *ErrorMinor)
 {
-    static Bool Initialised=FALSE;
+    static Bool Initialised = FALSE;
 
     if (!Initialised) {
-        Initialised=TRUE;
-        xf86AddDriver(&RENDITION, Module, 0);
-        LoaderRefSymLists(vgahwSymbols, ramdacSymbols,
-			  fbSymbols, xaaSymbols, ddcSymbols, int10Symbols,
-			  shadowfbSymbols, vbeSymbols, NULL);
-        return (pointer)TRUE;
+        Initialised = TRUE;
+        xf86AddDriver(&RENDITION, Module, 1);
+        return (pointer) TRUE;
     }
 
     if (ErrorMajor)
-        *ErrorMajor=LDR_ONCEONLY;
+        *ErrorMajor = LDR_ONCEONLY;
 
     return NULL;
 }
@@ -259,24 +243,6 @@ renditionSetup(pointer Module, pointer Options, int *ErrorMajor,
 #endif
 
 
-enum renditionTypes {
-    CHIP_RENDITION_V1000,
-    CHIP_RENDITION_V2x00
-};
-
-/* supported chipsets */
-static SymTabRec renditionChipsets[] = {
-    {CHIP_RENDITION_V1000, "V1000"},
-    {CHIP_RENDITION_V2x00, "V2x00"},
-    {-1,                   NULL}
-};
-
-static PciChipsets renditionPCIchipsets[] = {
-  { CHIP_RENDITION_V1000, PCI_CHIP_V1000, RES_SHARED_VGA },
-  { CHIP_RENDITION_V2x00, PCI_CHIP_V2x00, RES_SHARED_VGA },
-  { -1,                   -1,             RES_UNDEFINED }
-};
-
 /*
  * functions
  */
@@ -297,6 +263,49 @@ renditionIdentify(int flags)
 
 
 
+#ifdef XSERVER_LIBPCIACCESS
+static Bool
+renditionPciProbe(DriverPtr drv, int entity_num, struct pci_device *dev,
+		  intptr_t match_data)
+{
+    ScrnInfoPtr pScrn;
+
+
+    /* Allocate a ScrnInfoRec and claim the slot */
+    pScrn = xf86ConfigPciEntity(NULL, 0, entity_num, NULL, RES_SHARED_VGA,
+				NULL, NULL, NULL, NULL);
+    if (pScrn != NULL) {
+	renditionPtr pRendition;
+
+
+	pScrn->driverVersion = RENDITION_VERSION_CURRENT;
+	pScrn->driverName    = RENDITION_DRIVER_NAME;
+	pScrn->name          = RENDITION_NAME;
+	pScrn->Probe         = NULL;
+	pScrn->PreInit       = renditionPreInit;
+	pScrn->ScreenInit    = renditionScreenInit;
+	pScrn->SwitchMode    = renditionSwitchMode;
+	pScrn->AdjustFrame   = renditionAdjustFrame;
+	pScrn->EnterVT       = renditionEnterVT;
+	pScrn->LeaveVT       = renditionLeaveVT;
+	pScrn->FreeScreen    = renditionFreeScreen;
+	pScrn->ValidMode     = renditionValidMode;
+
+	/* allocate driver private structure */
+	pRendition = renditionGetRec(pScrn);
+	if (pRendition == NULL) {
+	    return FALSE;
+	}
+
+	pRendition->pEnt = xf86GetEntityInfo(entity_num);
+	pRendition->PciInfo = dev;
+    }
+
+    return (pScrn != NULL);
+}
+
+#else
+
 /*
  * This function is called once, at the start of the first server generation to
  * do a minimal probe for supported hardware.
@@ -354,7 +363,7 @@ renditionProbe(DriverPtr drv, int flags)
     }
     return foundScreen;
 }
-
+#endif
 
 #if 0
 static Bool
@@ -491,16 +500,20 @@ renditionPreInit(ScrnInfoPtr pScreenInfo, int flags)
     if (pScreenInfo->numEntities != 1)
 	return FALSE;
 
+#ifndef XSERVER_LIBPCIACCESS
     /* allocate driver private structure */
     if (!renditionGetRec(pScreenInfo))
         return FALSE;
+#endif
 
     pRendition=RENDITIONPTR(pScreenInfo);
 
+#ifndef XSERVER_LIBPCIACCESS
     /* Get the entity, and make sure it is PCI. */
     pRendition->pEnt = xf86GetEntityInfo(pScreenInfo->entityList[0]);
     if (pRendition->pEnt->location.type != BUS_PCI)
 	return FALSE;
+#endif
 
     if (flags & PROBE_DETECT) {
         ConfiguredMonitor = 
@@ -515,17 +528,17 @@ renditionPreInit(ScrnInfoPtr pScreenInfo, int flags)
     if (xf86LoadSubModule(pScreenInfo, "int10")){
         xf86Int10InfoPtr pInt=NULL;
 
-        xf86LoaderReqSymLists(int10Symbols, NULL);
-
         xf86DrvMsg(pScreenInfo->scrnIndex, X_INFO, "Initializing int10\n");
         pInt = xf86InitInt10(pRendition->pEnt->index);
         xf86FreeInt10(pInt);
     }
 
+#ifndef XSERVER_LIBPCIACCESS
     /* Find the PCI info for this screen */
     pRendition->PciInfo = xf86GetPciInfoForEntity(pRendition->pEnt->index);
     pRendition->pcitag= pciTag(pRendition->PciInfo->bus,
                pRendition->PciInfo->device, pRendition->PciInfo->func);
+#endif
 
     /*
      * XXX This could be refined if some VGA memory resources are not
@@ -557,7 +570,7 @@ renditionPreInit(ScrnInfoPtr pScreenInfo, int flags)
 
         case 15:
         {
-            if (PCI_CHIP_V1000 != pRendition->PciInfo->chipType) {
+            if (PCI_CHIP_V1000 == DEVICE_ID(pRendition->PciInfo)) {
                 xf86DrvMsg( pScreenInfo->scrnIndex, X_ERROR,
                         "Given depth (%d) is not supported by this chipset.\n",
                         pScreenInfo->depth);
@@ -594,8 +607,6 @@ renditionPreInit(ScrnInfoPtr pScreenInfo, int flags)
     if (!xf86LoadSubModule(pScreenInfo, "fb"))
       return FALSE;
 
-    xf86LoaderReqSymLists(fbSymbols, NULL);
-
     /* determine colour weights */
     pScreenInfo->rgbBits=8;
     
@@ -631,7 +642,7 @@ renditionPreInit(ScrnInfoPtr pScreenInfo, int flags)
     /* set various fields according to the given options */
     /* to be filled in <ml> */
 
-    if (PCI_CHIP_V1000==pRendition->PciInfo->chipType){
+    if (PCI_CHIP_V1000 == DEVICE_ID(pRendition->PciInfo)) {
       pRendition->board.chip=V1000_DEVICE;
     }
     else {
@@ -643,7 +654,6 @@ renditionPreInit(ScrnInfoPtr pScreenInfo, int flags)
     if (!xf86LoadSubModule(pScreenInfo, "vgahw")){
         return FALSE;
     }
-    xf86LoaderReqSymLists(vgahwSymbols, NULL);
 
     if (!vgaHWGetHWRec(pScreenInfo))
         return FALSE;
@@ -654,12 +664,19 @@ renditionPreInit(ScrnInfoPtr pScreenInfo, int flags)
 
     pRendition->board.accel=0;
     pRendition->board.vgaio_base = pvgaHW->PIOOffset;
-    pRendition->board.io_base =
-	pRendition->board.vgaio_base + pRendition->PciInfo->ioBase[1];
+    pRendition->board.io_base = pRendition->board.vgaio_base 
+#ifdef XSERVER_LIBPCIACCESS
+	+ pRendition->PciInfo->regions[1].base_addr;
+#else
+	+ pRendition->PciInfo->ioBase[1]
+#endif
+	;
     pRendition->board.mmio_base=0;
     pRendition->board.vmmio_base=0;
     pRendition->board.mem_size=0;
+#ifndef XSERVER_LIBPCIACCESS
     pRendition->board.mem_base=(vu8 *)pRendition->PciInfo->memBase[0];
+#endif
     pRendition->board.vmem_base=NULL;
     pRendition->board.init=0;
 
@@ -677,8 +694,14 @@ renditionPreInit(ScrnInfoPtr pScreenInfo, int flags)
 	       "Rendition %s @ %lx/%lx\n",
 	       renditionChipsets[pRendition->board.chip==V1000_DEVICE ? 0:1]
 	       .name,
+#ifdef XSERVER_LIBPCIACCESS
+	       pRendition->PciInfo->regions[1].base_addr,
+	       pRendition->PciInfo->regions[0].base_addr
+#else
 	       pRendition->PciInfo->ioBase[1],
-	       pRendition->PciInfo->memBase[0]);
+	       pRendition->PciInfo->memBase[0]
+#endif
+	       );
 
     /* First of all get a "clean" starting state */
     verite_resetboard(pScreenInfo);
@@ -730,7 +753,6 @@ renditionPreInit(ScrnInfoPtr pScreenInfo, int flags)
 		       "Oops, \"ShadowFB\" module loading failed, disabling ShadowFB!\n");
 	}
 	else{
-	    xf86LoaderReqSymLists(shadowfbSymbols, NULL);
 	    pRendition->board.shadowfb=TRUE;
 	    xf86DrvMsg(pScreenInfo->scrnIndex, X_INFO,
 		       "Using \"Shadow Framebuffer\"\n");
@@ -749,7 +771,6 @@ renditionPreInit(ScrnInfoPtr pScreenInfo, int flags)
       if (!xf86LoadSubModule(pScreenInfo, "ramdac")) {
 	return FALSE;
       }
-      xf86LoaderReqSymLists(ramdacSymbols, NULL);
     }
 
 #if 0
@@ -760,7 +781,6 @@ renditionPreInit(ScrnInfoPtr pScreenInfo, int flags)
 		   ("Loading of DDC library failed, skipping DDC-probe\n"));
       }
       else {
-	xf86LoaderReqSymLists(ddcSymbols, NULL);
 	pScreenInfo->monitor->DDC = renditionDDC(pScreenInfo);
       }
     }
@@ -777,7 +797,6 @@ renditionPreInit(ScrnInfoPtr pScreenInfo, int flags)
       }
       else {
 	  xf86MonPtr mon;
-	  xf86LoaderReqSymLists(ddcSymbols, NULL);
 	  mon = renditionProbeDDC(pScreenInfo, pRendition->pEnt->index);
 	  xf86PrintEDID(mon);
 	  xf86SetDDCproperties(pScreenInfo, mon);
@@ -854,7 +873,6 @@ renditionPreInit(ScrnInfoPtr pScreenInfo, int flags)
 	!pRendition->board.shadowfb) {
 	/* Load XAA if needed */
 	if (xf86LoadSubModule(pScreenInfo, "xaa")) {
-	    xf86LoaderReqSymLists(xaaSymbols, NULL);
 	    renditionMapMem(pScreenInfo);
   	    RENDITIONAccelPreInit (pScreenInfo);
 	    renditionUnmapMem(pScreenInfo);
@@ -1363,12 +1381,17 @@ renditionMapMem(ScrnInfoPtr pScreenInfo)
     Bool WriteCombine;
     int mapOption;
     renditionPtr pRendition = RENDITIONPTR(pScreenInfo);
+#ifdef XSERVER_LIBPCIACCESS
+    int err;
+#endif
 
 #ifdef DEBUG
     ErrorF("Mapping ...\n");
+#ifndef XSERVER_LIBPCIACCESS
     ErrorF("%d %d %d %x %d\n", pScreenInfo->scrnIndex, VIDMEM_FRAMEBUFFER, 
 	   pRendition->pcitag,
 	   pRendition->board.mem_base, pScreenInfo->videoRam * 1024);
+#endif
 #endif
 
     if (pRendition->board.chip == V1000_DEVICE){
@@ -1392,12 +1415,19 @@ renditionMapMem(ScrnInfoPtr pScreenInfo)
 	mapOption = VIDMEM_MMIO;
     }
 
+#ifdef XSERVER_LIBPCIACCESS
+    err = pci_device_map_region(pRendition->PciInfo, 0, TRUE);
+    pRendition->board.vmem_base = pRendition->PciInfo->regions[0].memory;
+
+    return (err == 0);
+#else
     pRendition->board.vmem_base=
         xf86MapPciMem(pScreenInfo->scrnIndex, mapOption,
 		      pRendition->pcitag,
 		      (unsigned long)pRendition->board.mem_base,
 		      pScreenInfo->videoRam * 1024);
     return TRUE;
+#endif
     
 #ifdef DEBUG0
     ErrorF("Done\n");
@@ -1407,12 +1437,20 @@ renditionMapMem(ScrnInfoPtr pScreenInfo)
 static Bool
 renditionUnmapMem(ScrnInfoPtr pScreenInfo)
 {
+  renditionPtr pRendition = RENDITIONPTR(pScreenInfo);
 #ifdef DEBUG
   ErrorF("Unmapping ...\n");
 #endif
+
+#ifndef XSERVER_LIBPCIACCESS
     xf86UnMapVidMem(pScreenInfo->scrnIndex,
-        RENDITIONPTR(pScreenInfo)->board.vmem_base, 
+        pRendition->board.vmem_base, 
 		    pScreenInfo->videoRam * 1024);
+#else
+    pci_device_unmap_range(pRendition->PciInfo, 
+			   pRendition->board.vmem_base,
+			   pScreenInfo->videoRam * 1024);
+#endif
     return TRUE;
 #ifdef DEBUG0
     ErrorF("Done\n");
@@ -1434,8 +1472,6 @@ renditionProbeDDC(ScrnInfoPtr pScreenInfo, int index)
   xf86MonPtr mon = NULL;
 
   if (xf86LoadSubModule(pScreenInfo, "vbe")) {
-    xf86LoaderReqSymLists(vbeSymbols, NULL);
-
     pVbe = VBEInit(NULL,index);
     mon = vbeDoEDID(pVbe, NULL);
     vbeFree(pVbe);
